# Copyright (c) 2012 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from PyQt4.QtGui import QAbstractButton, QLayout, QLineEdit

from ....model import adapt, Adapter, Instance, Model
from ....settings import ISettings

from ..visual_items import find_visual_item


class LocationEditorComponents(Model):
    """ The LocationEditorComponents class is an internal mixin for adapters
    that access the components of a location editor that embeds a
    :class:`~PyQt4.QtGui.QLineEdit` and a :class:`~PyQt4.QtGui.QAbstractButton`
    that implements a browse button.
    """

    # The line edit.
    line_edit = Instance(QLineEdit)

    @line_edit.default
    def line_edit(self):
        """ Invoked to return the line edit widget. """

        return find_visual_item(self.adaptee, QLineEdit)

    # The browse button.
    browse_button = Instance(QAbstractButton)

    @browse_button.default
    def browse_button(self):
        """ Invoked to return the add button. """

        return find_visual_item(self.adaptee, QAbstractButton)

    @classmethod
    def usable_editor(cls, item):
        """ Return True if an item can be used as an editor. """

        # The line edit must be an immediate child otherwise the criteria would
        # be far too wide.
        if find_visual_item(item, QLineEdit, recurse=False) is None:
            return False

        if find_visual_item(item, QAbstractButton) is None:
            return False

        return True


@adapt(QLayout, to=ISettings)
class QLayoutISettingsAdapter(Adapter, LocationEditorComponents):
    """ An adapter to implement ISettings for a location editor. """

    @classmethod
    def isadaptable(cls, adaptee):
        """ Determine if this adapter can adapt a particular layout.

        :param adaptee:
            is the layout.
        :return:
            ``True`` if the layout can be adapted.
        """

        return cls.usable_editor(adaptee)

    @ISettings.id.getter
    def id(self):
        """ Invoked to get the identifier. """

        return self.adaptee.objectName()

    def restore(self, settings_manager):
        """ Restore the settings obtained from a settings manager.

        :param settings_manager:
            is the settings manager.
        """

        value = settings_manager.read_value('location')
        if value is not None:
            self.line_edit.setText(value)

    def save(self, settings_manager):
        """ Save the settings to a settings manager.

        :param settings_manager:
            is the settings manager.
        """

        settings_manager.write_value('location', self.line_edit.text())
