# Copyright (c) 2012 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from PyQt4.QtCore import Qt
from PyQt4.QtGui import QMainWindow

from ....model import adapt, Adapter, observe, unadapted
from ....settings import ISettings
from ....ui import IAction, IActionCollection, IMainWindow, IMenuBar

from ..single_view_container_adapter import SingleViewContainerAdapter
from ..utils import as_QWidget, from_QWidget


@adapt(QMainWindow, to=IMainWindow)
class QMainWindowIMainWindowAdapter(SingleViewContainerAdapter):
    """ An adapter to implement IMainWindow for a QMainWindow. """

    # Map toolkit independent dock areas to the Qt specific equivalents.
    _area_map = {
            'left': Qt.LeftDockWidgetArea,
            'right': Qt.RightDockWidgetArea,
            'top': Qt.TopDockWidgetArea,
            'bottom': Qt.BottomDockWidgetArea}

    @observe('actions')
    def __actions_changed(self, change):
        """ Invoked when the list of actions changes. """

        menu_bar = self.adaptee.menuBar()

        if menu_bar is None:
            return

        # FIXME: Remove old actions.

        # Add any new actions.
        for action in change.new:
            if not IMenuBar(menu_bar).add_action(action):
                if isinstance(action, IAction):
                    raise ValueError(
                            "unable to position action '{0}'".format(
                                            IAction(action).id))

                raise ValueError(
                        "unable to position action collection '{0}'".format(
                                IActionCollection(action).id))

    @observe('docks')
    def __docks_changed(self, change):
        """ Invoked when the list of docks changes. """

        # FIXME: Remove old docks.

        # Add any new docks.
        for dock in change.new:
            # Convert the dock area.
            area = self._area_map[dock.area]

            # Add it to the main window.
            self.adaptee.addDockWidget(area, unadapted(dock))

            # Add any action.
            if dock.action is not None:
                dock.action.within = dock.within
                self.actions.append(dock.action)

    @IMainWindow.menu_bar.getter
    def menu_bar(self):
        """ Invoked to get the main window's menu bar. """

        return self.adaptee.menuBar()

    @menu_bar.setter
    def menu_bar(self, value):
        """ Invoked to set the main window's menu bar. """

        self.adaptee.setMenuBar(unadapted(value))

    @IMainWindow.view.getter
    def view(self):
        """ Invoked to get the main window's content. """

        return from_QWidget(self.adaptee.centralWidget())

    @view.setter
    def view(self, value):
        """ Invoked to set the main window's content. """

        self.adaptee.setCentralWidget(as_QWidget(value))

    def all_views(self):
        """ Create a generator that will return this view and any views
        contained in it.

        :return:
            the generator.
        """

        for dock in self.docks:
            for view in dock.all_views():
                yield view

        for view in super(QMainWindowIMainWindowAdapter, self).all_views():
            yield view


@adapt(QMainWindow, to=ISettings)
class QMainWindowISettingsAdapter(Adapter):
    """ An adapter to implement ISettings for a QMainWindow. """

    @ISettings.id.getter
    def id(self):
        """ Invoked to get the identifier. """

        return self.adaptee.objectName()

    def restore(self, settings_manager):
        """ Restore the settings obtained from a settings manager.

        :param settings_manager:
            is the settings manager.
        """

        widget = self.adaptee

        value = settings_manager.read_value('state')
        if value is not None:
            widget.restoreState(value)

        # Only apply the geometry if we are a top-level window.
        if widget.parent() is None:
            value = settings_manager.read_value('geom')
            if value is not None:
                widget.restoreGeometry(value)

    def save(self, settings_manager):
        """ Save the settings to a settings manager.

        :param settings_manager:
            is the settings manager.
        """

        widget = self.adaptee

        # Only save the geometry if we are a top-level window.
        if widget.parent() is None:
            settings_manager.write_value('geom', widget.saveGeometry())

        settings_manager.write_value('state', widget.saveState())
